"use strict";

exports.__esModule = true;
exports.default = void 0;
var _minim = require("minim");
var _EvaluationRealm = _interopRequireDefault(require("../EvaluationRealm.cjs"));
var _JSONPointerKeyError = _interopRequireDefault(require("../../../errors/JSONPointerKeyError.cjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
class MinimEvaluationRealm extends _EvaluationRealm.default {
  name = 'minim';
  isArray(node) {
    return node instanceof _minim.ArrayElement && !(node instanceof _minim.ObjectElement);
  }
  isObject(node) {
    return node instanceof _minim.ObjectElement;
  }
  sizeOf(node) {
    if (this.isArray(node) || this.isObject(node)) {
      return node.length;
    }
    return 0;
  }
  has(node, referenceToken) {
    if (this.isArray(node)) {
      return Number(referenceToken) < this.sizeOf(node);
    }
    if (this.isObject(node)) {
      const keys = node.keys();
      const uniqueKeys = new Set(keys);
      if (keys.length !== uniqueKeys.size) {
        throw new _JSONPointerKeyError.default(`Object key "${referenceToken}" is not unique — JSON Pointer requires unique member names`, {
          currentValue: node,
          referenceToken
        });
      }
      return node.hasKey(referenceToken);
    }
    return false;
  }
  evaluate(node, referenceToken) {
    if (this.isArray(node)) {
      return node.get(Number(referenceToken));
    }
    return node.get(referenceToken);
  }
}
var _default = exports.default = MinimEvaluationRealm;